# -*- coding: UTF-8 -*-

"""
Name: ffplay_file.py
Porpose: playback media file via ffplay
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2025 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Sep.18.2025
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
from threading import Thread
import subprocess
import platform
import wx
from pubsub import pub
from videomass.vdms_utils.utils import Popen
from videomass.vdms_io.io_tools import show_msg_notify
from videomass.vdms_io.make_filelog import make_log_template, tolog
if not platform.system() == 'Windows':
    import shlex


class FilePlay(Thread):
    """
    Playback local file executing ffplay media player.
    """
    def __init__(self, parent, param=''):
        """
        Don't use the "-stats" option here to avoid too much verbose
        output being shown in the message box as an error every time.

        """
        self.parent = parent  # is the top-level window (main_frame)
        get = wx.GetApp()
        self.appdata = get.appset
        self.param = param  # additional parameters if presentì
        self.logfile = os.path.join(self.appdata['logdir'], 'ffplay.log')
        make_log_template('ffplay.log', self.appdata['logdir'], mode="w")

        Thread.__init__(self)
        self.start()
    # ----------------------------------------------------------------#

    def run(self):
        """
        Executes ffplay command via subprocess.Popen .
        IMPORTANT: do not use class Popen from utils.py here,
        because 'info' flag do not work on MS-Windows using ffplay.
        This is fixed using shell=True flag.
        """
        cmd = (f'"{self.appdata["ffplay_cmd"]}" '
               f'{self.appdata["ffplay-default-args"]} '
               f'{self.appdata["ffplay_loglev"]} '
               f'{self.param}'
               )
        tolog(f'INFO: VIDEOMASS COMMAND: {cmd}', self.logfile,
              sep=True, wdate=True)

        if not platform.system() == 'Windows':
            cmd = shlex.split(cmd)
            info = None
            shell = False
        else:
            shell = True
            info = None
            # info = subprocess.STARTUPINFO()
            # info.dwFlags |= subprocess.SW_HIDE
        try:
            with subprocess.Popen(cmd,
                                  shell=shell,
                                  stderr=subprocess.PIPE,
                                  universal_newlines=True,
                                  encoding=self.appdata['encoding'],
                                  startupinfo=info,
                                  ) as proc:

                error = proc.communicate()[1]

                if error:
                    tolog(f"[FFPLAY] OUTPUT ERROR:\n{error}", self.logfile)
                    wx.CallAfter(show_msg_notify, self.parent,
                                 logname='ffplay.log')
                    return

        except OSError as err:
            tolog(f'[VIDEOMASS]: ERROR: {err}', self.logfile)
            wx.CallAfter(show_msg_notify, self.parent, logname='ffplay.log')
            # do not return here, go to END_PLAY directly

        wx.CallAfter(pub.sendMessage, "END_PLAY")


class FilePlayback_GetOutput(Thread):
    """
    Playback local file executing ffplay media player.
    This class differs from `FilePlay` in that it implements
    capturing and redirecting output messages from stderr.
    """
    def __init__(self, parent, param=''):
        """
        The `stats` argument allows ffplay to update output
        messages progressively. This allows parsing of the
        output for progress informations.

        """
        self.parent = parent  # is the top-level window (main_frame)
        get = wx.GetApp()
        self.appdata = get.appset
        self.param = param  # additional parameters if present
        self.stats = '-stats'
        self.logfile = os.path.join(self.appdata['logdir'], 'ffplay.log')
        # set initial file LOG:
        make_log_template('ffplay.log', self.appdata['logdir'], mode="w")

        Thread.__init__(self)
        self.start()
    # ----------------------------------------------------------------#

    def run(self):
        """
        Executes ffplay command via Popen from utils.py.
        """
        # time.sleep(.5)
        line = 'NO MESSAGE PROVIDED :-('
        cmd = (f'"{self.appdata["ffplay_cmd"]}" '
               f'{self.appdata["ffplay-default-args"]} '
               f'{self.appdata["ffplay_loglev"]} {self.stats} '
               f'{self.param}'
               )
        tolog(f'INFO: VIDEOMASS COMMAND: {cmd}', self.logfile,
              sep=True, wdate=True
              )
        if not platform.system() == 'Windows':
            cmd = shlex.split(cmd)
        try:
            with Popen(cmd,
                       stderr=subprocess.PIPE,
                       bufsize=1,
                       universal_newlines=True,
                       encoding=self.appdata['encoding'],
                       ) as proc:
                for line in proc.stderr:
                    wx.CallAfter(pub.sendMessage,
                                 "UPDATE_PLAY_COUNTER",
                                 output=line,
                                 status=0
                                 )
                if proc.wait():  # ..Failed
                    wx.CallAfter(pub.sendMessage,
                                 "UPDATE_PLAY_COUNTER",
                                 output=None,
                                 status=proc.wait(),
                                 )
                    tolog(f"[FFPLAY] ERROR:\n{line}", self.logfile)
                    wx.CallAfter(show_msg_notify, self.parent,
                                 logname='ffplay.log')

        except OSError as err:
            tolog(f'[VIDEOMASS]: ERROR: {err}', self.logfile)
            wx.CallAfter(show_msg_notify, self.parent, logname='ffplay.log')
            # do not return here, go to END_PLAY

        wx.CallAfter(pub.sendMessage, "END_PLAY")



# -*- coding: UTF-8 -*-
"""
Name: timeline.py
Porpose: Time trimming settings
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2025 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Sep.18.2025
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import sys
import wx
import wx.adv
from pubsub import pub
from videomass.vdms_utils.get_bmpfromsvg import get_bmp
from videomass.vdms_utils.utils import integer_to_time
from videomass.vdms_utils.utils import time_to_integer
from videomass.vdms_dialogs.widget_utils import NormalTransientPopup
from videomass.vdms_io.make_filelog import make_log_template
from videomass.vdms_threads.generic_task import FFmpegGenericTask
from videomass.vdms_io.io_tools import show_msg_notify
from videomass.vdms_threads.ffplay_file import FilePlayback_GetOutput


class Time_Selector(wx.Dialog):
    """
    fine-tune segments duration
    """
    def __init__(self, parent, clockstr, mode):
        """
        When this dialog is called, the values already set
        in the timeline panel are reproduced exactly here.
        """
        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)
        sizer_base = wx.BoxSizer(wx.VERTICAL)
        caption = 'No caption'
        if mode == 'start':
            caption = _('Start point adjustment')
        elif mode == 'duration':
            caption = _('End point adjustment')
        boxs = wx.StaticBox(self, wx.ID_ANY, 'HH:MM:SS.ms')
        staticbox1 = wx.StaticBoxSizer(boxs, wx.VERTICAL)
        boxsiz1 = wx.BoxSizer(wx.HORIZONTAL)
        staticbox1.Add(boxsiz1, 0, wx.ALL | wx.ALIGN_CENTER, 0)
        sizer_base.Add(staticbox1, 0, wx.ALL | wx.EXPAND, 5)

        self.box_hour = wx.SpinCtrl(self, wx.ID_ANY,
                                    value=f"{clockstr[0:2]}",
                                    min=0, max=23,
                                    style=wx.SP_ARROW_KEYS)

        boxsiz1.Add(self.box_hour, 0, wx.ALL | wx.ALIGN_CENTER, 5)
        lab1 = wx.StaticText(self, wx.ID_ANY, (":"))
        lab1.SetFont(wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
        boxsiz1.Add(lab1, 0, wx.ALIGN_CENTER, 5)

        self.box_min = wx.SpinCtrl(self, wx.ID_ANY,
                                   value=f"{clockstr[3:5]}",
                                   min=0, max=59,
                                   style=wx.SP_ARROW_KEYS)

        boxsiz1.Add(self.box_min, 0, wx.ALL | wx.ALIGN_CENTER, 5)
        lab2 = wx.StaticText(self, wx.ID_ANY, (":"))
        lab2.SetFont(wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
        boxsiz1.Add(lab2, 0, wx.ALIGN_CENTER, 5)

        self.box_sec = wx.SpinCtrl(self, wx.ID_ANY,
                                   value=f"{clockstr[6:8]}",
                                   min=0, max=59,
                                   style=wx.SP_ARROW_KEYS)

        boxsiz1.Add(self.box_sec, 0, wx.ALL | wx.ALIGN_CENTER, 5)
        lab3 = wx.StaticText(self, wx.ID_ANY, ("."))
        lab3.SetFont(wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
        boxsiz1.Add(lab3, 0, wx.ALIGN_CENTER, 5)

        self.box_mills = wx.SpinCtrl(self, wx.ID_ANY,
                                     value=f"{clockstr[9:12]}",
                                     min=000, max=999,
                                     style=wx.SP_ARROW_KEYS)
        boxsiz1.Add(self.box_mills, 0, wx.ALL | wx.ALIGN_CENTER, 5)
        # confirm buttons:
        gridexit = wx.BoxSizer(wx.HORIZONTAL)
        btn_close = wx.Button(self, wx.ID_CANCEL, "")
        gridexit.Add(btn_close, 0, wx.ALL, 5)
        btn_ok = wx.Button(self, wx.ID_OK, "")
        gridexit.Add(btn_ok, 0, wx.ALL, 5)
        sizer_base.Add(gridexit, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, 0)

        self.SetSizer(sizer_base)
        sizer_base.Fit(self)
        self.Layout()

        # ----------------------Properties ----------------------#

        self.SetTitle(caption)
        self.box_hour.SetToolTip(_("Hours"))
        self.box_min.SetToolTip(_("Minutes"))
        self.box_sec.SetToolTip(_("Seconds"))
        self.box_mills.SetToolTip(_("Milliseconds"))

        # ----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_BUTTON, self.on_close, btn_close)
        self.Bind(wx.EVT_BUTTON, self.on_ok, btn_ok)

    # ----------------------Event handler (callback)----------------------#

    def on_close(self, event):
        """
        Close this dialog without saving anything
        """
        event.Skip()  # need if destroy from parent
    # ------------------------------------------------------------------#

    def on_ok(self, event):
        """
        Confirm event
        """
        event.Skip()
    # ------------------------------------------------------------------#

    def getvalue(self):
        """
        This method return values via the getvalue() interface
        from the caller.
        """
        val = (f'{str(self.box_hour.GetValue()).zfill(2)}:'
               f'{str(self.box_min.GetValue()).zfill(2)}:'
               f'{str(self.box_sec.GetValue()).zfill(2)}.'
               f'{str(self.box_mills.GetValue()).zfill(3)}'
               )
        return val, time_to_integer(val)


class Float_TL(wx.MiniFrame):
    """
    A graphical representation of the Float_TL object,
    a floating bar (ruler) for trimming and slicing time
    segments from media duration.

    """
    get = wx.GetApp()
    ICONS = get.iconset
    # COLORSCHEME = get.appset['colorscheme']
    OS = get.appset['ostype']
    LOGDIR = get.appset['logdir']
    TMPROOT = os.path.join(get.appset['cachedir'], 'tmp', 'Waveforms')
    TMPSRC = os.path.join(TMPROOT, 'tmpsrc')
    os.makedirs(TMPSRC, mode=0o777, exist_ok=True)

    # Used Colours
    YELLOW = '#bd9f00'  # for warnings
    BLACK = '#060505'  # black for background status bar
    ORANGE = '#f28924'  # for errors and warnings
    LGREEN = '#52EE7D'
    RULER_BKGRD = '#84D2C9'  # CYAN for ruler background
    TEXTDEF = '#B1F2E8'  # Light CYAN
    SELECTION = (100, 250, 144, 100)  # Light CYAN
    # SELECTION = '#B1F2E8'  # Light CYAN
    DELIMITER_COLOR = '#009DCB'  # Azure for margin selection
    TEXT_PEN_COLOR = '#020D0F'  # black for draw lines
    DURATION_START = '#E95420'  # Light orange for duration/start indicators
    READMEGREEN = '#52ee7d'  # readme btn background
    READMEBLACK = '#1f1f1f'  # readme btn foreground

    # ruler and panel specifications constants
    RW = 900  # ruler width
    RM = 0  # ruler margin
    PW = 906  # panel width
    PH = 80  # panel height

    def __init__(self, parent):
        """

        self.pix: scale pixels to milliseconds
        self.milliseconds: int(milliseconds)
        self.duration: see parent attr.
        self.bar_w: END point value in pixel
        self.bar_x: START point value in pixel
        """
        icons = Float_TL.ICONS

        if 'wx.svg' in sys.modules:  # only available in wx version 4.1 to up
            bmp_reset = get_bmp(icons['clear'], ((16, 16)))
            bmp_tstart = get_bmp(icons['start_time'], ((16, 16)))
            bmp_tend = get_bmp(icons['end_time'], ((16, 16)))
            bmp_wave = get_bmp(icons['waveform'], ((16, 16)))
            bmp_play = get_bmp(icons['playback'], ((16, 16)))
        else:
            bmp_reset = wx.Bitmap(icons['clear'], wx.BITMAP_TYPE_ANY)
            bmp_tstart = wx.Bitmap(icons['start_time'], wx.BITMAP_TYPE_ANY)
            bmp_tend = wx.Bitmap(icons['end_time'], wx.BITMAP_TYPE_ANY)
            bmp_wave = wx.Bitmap(icons['waveform'], wx.BITMAP_TYPE_ANY)
            bmp_play = wx.Bitmap(icons['playback'], wx.BITMAP_TYPE_ANY)

        self.parent = parent
        self.duration = self.parent.duration
        self.overalltime = '00:00:00.000'
        self.milliseconds = 86399999  # 23:59:59:999
        self.clock_start = '00:00:00.000'  # seek position
        self.clock_end = '00:00:00.000'
        self.mills_end = 0  # end point in milliseconds
        self.mills_start = 0  # stat point in milliseconds
        self.pix = Float_TL.RW / self.milliseconds  # scale factor
        self.bar_w = 0  # End point value in pixel
        self.bar_x = 0  # Start point value in pixel
        self.pointpx = [0, 0]  # mouse points (see on_move(), on_leftdown())
        self.sourcedur = _('No source duration:')
        self.filename = None  # selected filename on file list
        self.frame = None  # waveform image located on cache dir
        self.invalidselection = False  # booleaan reference
        self.playpoint = 0  # `x` point pixel representation

        wx.MiniFrame.__init__(self, parent, -1, style=wx.CAPTION | wx.CLOSE_BOX
                              | wx.SYSTEM_MENU | wx.FRAME_FLOAT_ON_PARENT
                              | wx.RESIZE_BORDER
                              )
        self.panelbase = wx.Panel(self, wx.ID_ANY, style=wx.TAB_TRAVERSAL
                                  | wx.BORDER_THEME)
        sizer_base = wx.BoxSizer(wx.VERTICAL)
        self.panelruler = wx.Panel(self.panelbase, wx.ID_ANY,
                                   size=(Float_TL.PW, Float_TL.PH),
                                   style=wx.BORDER_SUNKEN,
                                   )
        sizer_base.Add(self.panelruler, 0, wx.ALL | wx.CENTRE, 2)
        sizer_btns = wx.BoxSizer(wx.HORIZONTAL)
        sizer_btns = wx.FlexGridSizer(0, 7, 0, 0)
        sizer_base.Add(sizer_btns, 0, wx.ALL | wx.CENTRE, 8)
        self.btn_play = wx.Button(self.panelbase, wx.ID_ANY, "", size=(40, -1))
        self.btn_play.SetBitmap(bmp_play, wx.LEFT)
        sizer_btns.Add(self.btn_play, 0, wx.RIGHT | wx.CENTRE, 20)

        self.counterplay = wx.StaticText(self.panelbase, wx.ID_ANY,
                                         ("00:00:00.000"))
        self.counterplay.SetFont(wx.Font(12, wx.DEFAULT, wx.NORMAL,
                                         wx.BOLD, 0, ""))
        sizer_btns.Add(self.counterplay, 0, wx.RIGHT | wx.ALIGN_CENTRE_VERTICAL
                       | wx.ALIGN_CENTRE_HORIZONTAL, 20)
        if Float_TL.OS == 'Windows':
            self.counterplay.Hide()
            self.btn_play.Hide()
        self.btn_tstart = wx.Button(self.panelbase, wx.ID_ANY, "",
                                    size=(40, -1))
        self.btn_tstart.SetBitmap(bmp_tstart, wx.LEFT)
        sizer_btns.Add(self.btn_tstart, 0, wx.CENTRE, 5)
        self.btn_reset = wx.Button(self.panelbase, wx.ID_ANY, "",
                                   size=(40, -1))
        self.btn_reset.SetBitmap(bmp_reset, wx.LEFT)
        sizer_btns.Add(self.btn_reset, 0, wx.LEFT | wx.CENTRE, 5)
        self.btn_tend = wx.Button(self.panelbase, wx.ID_ANY, "",
                                  size=(40, -1))
        self.btn_tend.SetBitmap(bmp_tend, wx.LEFT)
        sizer_btns.Add(self.btn_tend, 0, wx.LEFT | wx.CENTRE, 5)
        self.btn_wave = wx.ToggleButton(self.panelbase, wx.ID_ANY, _("Wave"),
                                        size=(-1, -1))
        self.btn_wave.SetBitmap(bmp_wave, wx.LEFT)
        sizer_btns.Add(self.btn_wave, 0, wx.LEFT | wx.CENTRE, 20)
        btn_readme = wx.Button(self.panelbase, wx.ID_ANY, _("Read me"),
                               size=(-1, -1))
        btn_readme.SetBackgroundColour(wx.Colour(Float_TL.READMEGREEN))
        btn_readme.SetForegroundColour(wx.Colour(Float_TL.READMEBLACK))
        sizer_btns.Add(btn_readme, 0, wx.LEFT | wx.CENTRE, 20)

        # ----------------------Properties ----------------------#
        self.panelruler.SetBackgroundColour(wx.Colour(Float_TL.RULER_BKGRD))
        # self.panelruler.SetBackgroundColour(wx.Colour(Float_TL.BLACK))
        self.SetTitle("Timeline Editor")
        self.sb = self.CreateStatusBar(1)
        msg = _('{0} {1}  |  Segment Duration: {2}'
                ).format(self.sourcedur, self.overalltime, '00:00:00.000')
        self.statusbar_msg(msg, None)

        # ----------------------Layout----------------------#
        self.panelbase.SetSizer(sizer_base)
        sizer_base.Fit(self)
        self.Layout()

        self.SetMinSize((920, 190))

        if Float_TL.OS == 'Linux':
            self.font_med = wx.Font(9, wx.DEFAULT, wx.NORMAL, wx.BOLD)
        elif Float_TL.OS == 'Windows':
            self.font_med = wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.BOLD)
        elif Float_TL.OS == 'Darwin':
            self.font_med = wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.BOLD)
        else:
            self.font_med = wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.BOLD)

        # ---------------------- disable all controls by default
        self.panelbase.Disable()

        # ---------------------- Tooltips
        tip = _('End adjustment')
        self.btn_tend.SetToolTip(tip)
        tip = _('Start adjustment')
        self.btn_tstart.SetToolTip(tip)
        tip = (_('Toggles the display of the audio waveform in the '
                 'background ruler'))
        self.btn_wave.SetToolTip(tip)
        tip = _('Reset segment on timeline')
        self.btn_reset.SetToolTip(tip)
        tip = _('Timeline Editor Usage')
        btn_readme.SetToolTip(tip)
        tip = _('Play segment')
        self.btn_play.SetToolTip(tip)

        # ----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_TOGGLEBUTTON, self.on_load_waveform, self.btn_wave)
        self.Bind(wx.EVT_BUTTON,
                  lambda event: self.on_set_pos(event, mode='duration'),
                  self.btn_tend)
        self.Bind(wx.EVT_BUTTON,
                  lambda event: self.on_set_pos(event, mode='start'),
                  self.btn_tstart)
        self.Bind(wx.EVT_BUTTON, self.on_trim_time_reset, self.btn_reset)
        self.Bind(wx.EVT_BUTTON, self.on_help, btn_readme)
        self.Bind(wx.EVT_BUTTON, self.on_play_segment, self.btn_play)
        self.panelruler.Bind(wx.EVT_PAINT, self.OnPaint)
        self.panelruler.Bind(wx.EVT_LEFT_DOWN, self.on_leftdown)
        self.panelruler.Bind(wx.EVT_LEFT_UP, self.on_leftup)
        self.panelruler.Bind(wx.EVT_LEFT_DCLICK, self.on_set_pos)
        self.panelruler.Bind(wx.EVT_MOTION, self.on_move)
        self.Bind(wx.EVT_CLOSE, self.on_close)

        pub.subscribe(self.set_values, "RESET_ON_CHANGED_LIST")
        pub.subscribe(self.update_counter_thread, "UPDATE_PLAY_COUNTER")
        pub.subscribe(self.end_playback_thread, "END_PLAY")

    def file_selection(self):
        """
        Defines the selected file as filename. Returns an object
        of type list (str('selected file name'), int(index)).
        Returns None type if no file is selected.
        """
        if not self.parent.filedropselected:
            wx.MessageBox(_("Have to select an item in the file list first"),
                          'Videomass', wx.ICON_INFORMATION, self)
            return None
        self.filename = self.parent.filedropselected
        return (self.filename, self.parent.file_src.index(self.filename))
    # ------------------------------------------------------------------#

    def make_frame_from_file(self):
        """
        This method is responsible for making available
        an audio waveform frame from a selected file.
        Return None type if error, str(frame) as pathname otherwise.
        """
        logfile = make_log_template('generic_task.log', Float_TL.LOGDIR,
                                    mode="w")
        name = os.path.splitext(os.path.basename(self.filename))[0]
        self.frame = os.path.join(f'{Float_TL.TMPSRC}', f'{name}.png')  # image

        arg = (f'-i "{self.filename}" -filter_complex '
               f'"showwavespic=s={Float_TL.RW + 10}x{Float_TL.PH}'
               f':colors=white:split_channels=1" -update 1 '
               f'-frames:v 1 "{self.frame}"')
        thread = FFmpegGenericTask(arg, 'Timeline Waveform', logfile)
        thread.join()  # wait end thread
        error = thread.status
        if error:
            self.frame = None
            show_msg_notify(self.GetParent(),
                            logname=os.path.basename(logfile))
        return self.frame
    # ------------------------------------------------------------------#

    def get_audio_stream(self, fileselected):
        """
        Given a selected media file (object of type `file_selection()`),
        it evaluates whether it contains any audio streams.
        If no audio streams Returns None, True otherwise.
        """
        selected = self.parent.data_files[fileselected[1]].get('streams')
        isaudio = [a for a in selected if 'audio' in a.get('codec_type')]

        if isaudio:
            return True
        return None

    # ----------------------Event handler (callbacks)----------------------#

    def on_play_segment(self, event):
        """
        Plays a selected segment of a file.
        """
        if not self.file_selection():
            return

        if self.sourcedur == _('No source duration:'):
            wx.MessageBox(_('Invalid file format for playback.'),
                          _('Videomass - Warning!'), wx.ICON_WARNING, self)
            return

        if self.parent.time_seq == "":
            wx.MessageBox(_('No segment selected in the timeline yet.'),
                          _('Videomass - Warning!'), wx.ICON_WARNING, self)
            return

        if self.invalidselection:
            wx.MessageBox(_('Invalid segment selection!\n«End» cursor must be '
                            'moved to the right side of the «Start» cursor.'),
                          _('Videomass - Error!'), wx.ICON_ERROR, self)
            return

        if self.parent.checktimestamp:
            scalef, tstamp = 'scale=w=360:h=-1,', f'{self.parent.cmdtimestamp}'
        else:
            scalef, tstamp = 'scale=w=360:h=-1', ''

        args = (f'-autoexit -i "{self.file_selection()[0]}" '
                f'{self.parent.time_seq} -vf "{scalef}{tstamp}"')

        try:
            with open(self.file_selection()[0], encoding='utf-8'):
                FilePlayback_GetOutput(self.GetParent(), args)
        except IOError:
            wx.MessageBox(_("Invalid or unsupported file:  %s") % (filepath),
                          "Videomass", wx.ICON_EXCLAMATION, self)
            return

        self.btn_play.Disable()
        self.btn_tstart.Disable()
        self.btn_reset.Disable()
        self.btn_tend.Disable()
        self.panelruler.Disable()
        self.playpoint = round(self.bar_x)
    # ----------------------------------------------------------------------

    def update_counter_thread(self, output, status):
        """
        This method uses pub/sub protocol subscribing "UPDATE_PLAY_COUNTER"
        message. It is called by `FilePlay` class thread to update
        the output lines of the ffplay stderr.
        """
        if status:
            return

        timemils = 0.0
        if output:
            line = output.split()
            try:
                timemils = float(line[0]) * 1000
                res = integer_to_time(round(timemils))
                self.counterplay.SetLabel(res)
                self.onRedraw(wx.ClientDC(self.panelruler))
                scaletopix = round(self.pix / (Float_TL.RW
                                               / timemils) * Float_TL.RW)
                pixpos = scaletopix
                self.playpoint = pixpos

            except (ValueError, ZeroDivisionError):
                self.counterplay.SetLabel('N/A')
    # ----------------------------------------------------------------------

    def end_playback_thread(self):
        """
        This method uses pub/sub protocol subscribing "END_PLAY" message.
        It is called by `FilePlay` class thread to indicate the end of the
        thread.
        """
        self.panelruler.Enable()
        self.btn_play.Enable()
        self.btn_tstart.Enable()
        self.btn_reset.Enable()
        self.btn_tend.Enable()
        self.playpoint = 0
    # ----------------------------------------------------------------------

    def on_load_waveform(self, event):
        """
        Event on toggle waveform button
        """
        if self.btn_wave.GetValue() is True:
            if not self.file_selection():
                self.btn_wave.SetValue(False)
                return

            if not self.get_audio_stream(self.file_selection()):
                wx.MessageBox(_('Unable to create waveform.\nThe selected '
                                'source file does not contain any audio '
                                'streams:\n"{}"'
                                ).format(self.file_selection()[0]),
                              _('Videomass - Warning!'), wx.ICON_WARNING, self)
                self.btn_wave.SetValue(False)
                return

            if not self.make_frame_from_file():
                self.btn_wave.SetValue(False)
                return
        else:
            self.frame = None

        self.onRedraw(wx.ClientDC(self.panelruler))
    # ----------------------------------------------------------------------

    def on_help(self, event):
        """
        Event clicking on button Read me.
        """
        msg = (_('The timeline editor allows you to trim slices of time on '
                 'selected files.\nThe time format used to report durations '
                 'is expressed in hours, minutes,\nseconds and milliseconds '
                 '(HH:MM:SS.ms).\n\nThe «Start»/«End» duration values always '
                 'refer to the initial position of\na overall duration. '
                 'The overall duration of a file and the duration of a\n'
                 'selected segment (including warnings) are displayed on the '
                 'timeline\nstatus bar.\n\nPlease note that actions such as '
                 'importing new files, selecting, deselecting,\ndeleting, and '
                 'sorting items will reset the timeline to its default '
                 'values.\n\nIf no source file, the duration values ​​will be '
                 'set to the default value\n(00:00:00.000) and all timeline '
                 'controls will be disabled.\nThis behavior also applies when '
                 'deselecting files.'))

        win = NormalTransientPopup(self,
                                   wx.SIMPLE_BORDER,
                                   msg,
                                   Float_TL.READMEGREEN,
                                   Float_TL.READMEBLACK)

        # Show the popup right below or above the button
        # depending on available screen space...
        btn = event.GetEventObject()
        pos = btn.ClientToScreen((0, 0))
        sz = btn.GetSize()
        win.Position(pos, (0, sz[1]))

        win.Popup()
    # --------------------------------------------------------------#

    def on_trim_time_reset(self, event):
        """
        Reset all timeline values to default.
        """
        self.pix = Float_TL.RW / self.milliseconds
        self.bar_w = 0
        self.bar_x = 0
        self.mills_end = 0
        self.clock_end = '00:00:00.000'
        self.mills_start = 0
        self.clock_start = '00:00:00.000'
        msg = _('{0} {1}  |  Segment Duration: {2}'
                ).format(self.sourcedur, self.overalltime, '00:00:00.000')
        self.counterplay.SetLabel('00:00:00.000')
        self.statusbar_msg(msg, None)
        self.parent.time_seq = ""
        self.onRedraw(wx.ClientDC(self.panelruler))
    # ------------------------------------------------------------------#

    def set_values(self, msg):
        """
        Any change to the file list in the drag panel reset
        timeline data. This method is called using pub/sub protocol
        subscribing "RESET_ON_CHANGED_LIST" by adding, selecting or
        removing imported files (see`filedrop.py`).
        """
        self.sourcedur = _('No source duration:')
        if msg is None:
            self.milliseconds = 86399999
            self.overalltime = '00:00:00.000'
            self.panelbase.Disable()
        else:
            if self.duration[msg] < 100:
                self.milliseconds = 86399999
                self.panelbase.Disable()
                self.overalltime = '00:00:00.000'
            else:
                self.milliseconds = self.duration[msg]
                self.sourcedur = _('Source duration:')
                self.panelbase.Enable()
                self.overalltime = integer_to_time(self.milliseconds)

        self.btn_wave.SetValue(False)
        self.frame = None
        self.on_trim_time_reset(None)
    # ------------------------------------------------------------------#

    def set_time_seq(self, isset=True):
        """
        Set parent time_seq attr.
        """
        if isset:
            dur = integer_to_time(self.mills_end - self.mills_start)
            self.parent.time_seq = f"-ss {self.clock_start} -t {dur}"
            msg = _('{0} {1}  |  Segment Duration: {2}'
                    ).format(self.sourcedur, self.overalltime, dur)
            self.statusbar_msg(f'{msg}', None)
        else:
            self.parent.time_seq = ""
            msg = _('{0} {1}  |  Segment Duration: {2}'
                    ).format(self.sourcedur, self.overalltime, '00:00:00.000')
            self.statusbar_msg(f'{msg}', None)
    # ------------------------------------------------------------------#

    def set_coordinates(self):
        """
        Set data for x axis rectangle selection and
        call onRedraw.
        """
        if self.pointpx[1] > 30:
            self.bar_w = self.pointpx[0]
            self.mills_end = int(round(self.bar_w / self.pix))
            self.clock_end = integer_to_time(self.mills_end)
            self.onRedraw(wx.ClientDC(self.panelruler))

        elif self.pointpx[1] < 30:
            self.bar_x = self.pointpx[0]
            self.mills_start = int(round(self.bar_x / self.pix))
            self.clock_start = integer_to_time(self.mills_start)
            self.onRedraw(wx.ClientDC(self.panelruler))
    # ------------------------------------------------------------------#

    def on_leftdown(self, event):
        """
        Event on clicking the left mouse button
        """
        self.panelruler.SetCursor(wx.Cursor(wx.CURSOR_HAND))
        self.pointpx = event.GetPosition()
        if self.pointpx[0] >= Float_TL.RW:
            return
        if self.pointpx[0] <= 0:
            return
        self.set_coordinates()
    # ------------------------------------------------------------------#

    def on_leftup(self, event):
        """
        Event on releasing the left mouse button

        """
        self.panelruler.SetCursor(wx.Cursor(wx.CURSOR_ARROW))

        if self.bar_w == 0 and self.bar_x == 0:
            self.set_time_seq(isset=False)
            return

        if (self.bar_w <= self.bar_x or self.mills_end > self.milliseconds
           or self.mills_start < 0):
            msg = _('WARNING: Invalid segment selection, out of range.')
            self.statusbar_msg(f'{msg}', Float_TL.ORANGE, Float_TL.BLACK)
            return

        self.set_time_seq(isset=True)
    # ------------------------------------------------------------------#

    def on_set_pos(self, event, mode=None):
        """
        Event on mouse double click right button.
        Note that pixpos variable is the KEY
        """
        if mode == 'duration':
            clock, mode = self.clock_end, 'duration'
        elif mode == 'start':
            clock, mode = self.clock_start, 'start'
        elif self.pointpx[1] > 30:
            clock, mode = self.clock_end, 'duration'
        elif self.pointpx[1] < 30:
            clock, mode = self.clock_start, 'start'
        else:
            return

        with Time_Selector(self, clock, mode) as selector:
            if selector.ShowModal() == wx.ID_OK:
                data = selector.getvalue()
                # pixpos: convert to new `x` point pixel
                pixpos = self.pix / (Float_TL.RW / data[1]) * Float_TL.RW
                self.pointpx[0] = round(pixpos)  # set x point
                if mode == 'start':
                    self.bar_x = pixpos
                    self.mills_start = data[1]
                    self.clock_start = data[0]
                elif mode == 'duration':
                    self.bar_w = pixpos
                    self.mills_end = data[1]
                    self.clock_end = data[0]

                self.onRedraw(wx.ClientDC(self.panelruler))
                self.on_leftup(None)
    # ------------------------------------------------------------------#

    def on_move(self, event):
        """
        Mouse event when moving it on the panel bar
        """
        if event.Dragging() and event.LeftIsDown():
            self.pointpx = event.GetPosition()
            if self.pointpx[1] > 30:
                if self.mills_end == self.milliseconds:
                    return
            elif self.pointpx[1] < 30:
                if self.mills_start == 0:
                    return
            dur = integer_to_time(self.mills_end - self.mills_start)
            msg = _('{0} {1}  |  Segment Duration: {2}'
                    ).format(self.sourcedur, self.overalltime, dur)
            self.statusbar_msg(f'{msg}', None)
            self.set_coordinates()
    # ------------------------------------------------------------------#

    def OnPaint(self, event):
        """
        wx.PaintDC event
        """
        dc = wx.PaintDC(self.panelruler)  # draw window boundary
        self.panelruler.Refresh()  # needed on wayland to make it work
        self.onRedraw(dc)
    # ------------------------------------------------------------------#

    def onRedraw(self, dc):
        """
        Draw ruler and update the selection rectangle
        (a semi-transparent background rectangle upon a ruler)
        """
        msg = (_('Start by dragging the handles in the left corners or\n'
                 'click the Start/End adjustment buttons for fine tuning.'))
        if Float_TL.OS == 'Windows':
            self.panelruler.SetDoubleBuffered(True)  # prevents flickers
        dc.Clear()

        # set start/end text colors
        if self.bar_w == 0 and self.bar_x == 0:
            self.invalidselection = False
            selcolor, textcolor = Float_TL.SELECTION, Float_TL.TEXTDEF
            dc.SetTextForeground(Float_TL.DURATION_START)
            # w = dc.GetTextExtent(msg)[0]
            dc.DrawText(msg, 310, 20)
        elif (self.bar_x >= self.bar_w or self.pointpx[0] > Float_TL.RW
              or self.pointpx[0] < 0):
            self.invalidselection = True
            selcolor, textcolor = wx.RED, wx.YELLOW
        else:
            self.invalidselection = False
            selcolor, textcolor = Float_TL.SELECTION, Float_TL.DURATION_START

        if self.frame and Float_TL.OS != 'Windows':
            bmp = wx.Image(self.frame, wx.BITMAP_TYPE_ANY).ConvertToBitmap()
            dc.DrawBitmap(bmp, 0, 0, True)

        self.text_time_indicator(dc, textcolor, selcolor)
        self.ruler_notches(dc)

        if self.playpoint:
            self.move_play_cursor(dc)

        if self.frame and Float_TL.OS == 'Windows':
            bmp = wx.Image(self.frame, wx.BITMAP_TYPE_ANY).ConvertToBitmap()
            dc.DrawBitmap(bmp, 0, 0, True)
    # ------------------------------------------------------------------#

    def text_time_indicator(self, dc, textcolor, selcolor):
        """
        Make text indicator & drag handles next to the boundaries
        e.g start/duration delimiters.
        """
        dc.SetPen(wx.Pen(Float_TL.DELIMITER_COLOR, 1, wx.PENSTYLE_SOLID))
        bar_w, bar_x = round(self.bar_w), round(self.bar_x)
        dc.SetBrush(wx.Brush(selcolor, wx.BRUSHSTYLE_SOLID))
        dc.DrawRectangle(bar_x, -2, bar_w - bar_x, 80)

        dc.SetFont(self.font_med)
        txt_s = _('Start')
        txt_d = _('End')
        dc.SetPen(wx.Pen(Float_TL.DELIMITER_COLOR, 2, wx.PENSTYLE_SOLID))
        dc.SetBrush(wx.Brush(Float_TL.DELIMITER_COLOR))

        # Make start txt
        txt1 = f'{txt_s}  {self.clock_start}'
        dc.SetTextForeground(textcolor)
        w = dc.GetTextExtent(txt1)[0]

        if w > bar_x:
            dc.DrawText(txt1, bar_x + 3, 20)
            dc.DrawRectangle(bar_x, 1, 7, 9)
        else:
            dc.DrawText(txt1, bar_x - w - 2, 20)
            dc.DrawRectangle(bar_x - 5, 1, 7, 9)

        # Make duration txt
        txt2 = f'{txt_d}  {self.clock_end}'
        w = dc.GetTextExtent(txt2)[0]
        if w < Float_TL.RW - bar_w:
            dc.DrawText(txt2, bar_w + 1, 42)
            dc.DrawRectangle(bar_w - 1, 68, 7, 10)
        else:
            dc.DrawText(txt2, bar_w - w - 5, 42)
            dc.DrawRectangle(bar_w - 6, 68, 7, 10)
    # ------------------------------------------------------------------#

    def ruler_notches(self, dc):
        """
        Make ruler delimiters sign
        """
        dc.SetPen(wx.Pen(Float_TL.TEXT_PEN_COLOR))
        dc.SetTextForeground(Float_TL.TEXT_PEN_COLOR)

        for i in range(Float_TL.RW):
            if not i % 600:
                dc.DrawLine(i + Float_TL.RM, 0, i + Float_TL.RM, 10)
            elif not i % 300:
                dc.DrawLine(i + Float_TL.RM, 0, i + Float_TL.RM, 10)
            elif not i % 150:
                dc.DrawLine(i + Float_TL.RM, 0, i + Float_TL.RM, 10)
            elif not i % 25:
                dc.DrawLine(i + Float_TL.RM, 0, i + Float_TL.RM, 5)
        dc.DrawLine(i, 0, i, 10)
    # ------------------------------------------------------------------#

    def move_play_cursor(self, dc):
        """
        Experimental, only for future implementations.
        ------------------------------------------
        This method should create a vertical colored cursor that
        slides from the start position to the end position as a
        file plays.
        USAGE EXAMPLE:
        self.playpoint = 100
        for i in range(10):
            self.onRedraw(wx.ClientDC(self.panelruler))
            self.playpoint += 50
        """
        dc.SetBrush(wx.Brush(wx.RED, wx.BRUSHSTYLE_SOLID))
        dc.SetPen(wx.Pen(wx.RED))
        dc.DrawRectangle(round(self.playpoint), 0, 3, 100)
        self.Update()
    # ------------------------------------------------------------------#

    def statusbar_msg(self, msg, bcolor, fcolor=None):
        """
        Set the status-bar message and color.
        See main_frame docstrings for details.
        """
        if Float_TL.OS == 'Linux':
            if not bcolor:
                self.sb.SetBackgroundColour(wx.NullColour)
                self.sb.SetForegroundColour(wx.NullColour)
            else:
                self.sb.SetBackgroundColour(bcolor)
                self.sb.SetForegroundColour(fcolor)

        if Float_TL.OS == 'Windows':
            self.sb.SetDoubleBuffered(True)  # prevents flickers
        self.sb.SetStatusText(msg)
        self.sb.Refresh()
    # ------------------------------------------------------------------#

    def on_close(self, event):
        """
        Hide this frame
        """
        self.parent.menuBar.Check(self.parent.viewtimeline.GetId(), False)
        self.Hide()



# -*- coding: UTF-8 -*-
"""
FileName: acodecs.py
Porpose: Contains audio functionality for A/V Conversions
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2025 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Sep.16.2025
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import sys
import wx
import wx.lib.scrolledpanel as scrolled
import wx.lib.agw.floatspin as FS
from videomass.vdms_utils.get_bmpfromsvg import get_bmp
from videomass.vdms_dialogs.audioproperties import AudioProperties
from videomass.vdms_utils.utils import get_volume_data
from videomass.vdms_io.io_tools import volume_detect_process
from videomass.vdms_dialogs.shownormlist import AudioVolNormal
from videomass.vdms_threads.ffplay_file import FilePlay


class AudioEncoders(scrolled.ScrolledPanel):
    """
    This scroll panel implements audio controls
    for A/V Conversions.
    """
    VIOLET = '#D64E93'
    # Namings in the audio codec selection on audio radio box:
    ACODECS = {('Auto'): (""),
               ('PCM'): ("pcm_s16le"),
               ('FLAC'): ("flac"),
               ('AAC'): ("aac"),
               ('ALAC'): ("alac"),
               ('AC3'): ("ac3"),
               ('VORBIS'): ("libvorbis"),
               ('LAME'): ("libmp3lame"),
               ('OPUS'): ("libopus"),
               ('Copy'): ("copy"),
               ('No Audio'): ("-an")
               }
    # compatibility between video formats and related audio codecs:
    AV_FORMATS = {('avi'): ('default', 'wav', None, None, None, 'ac3', None,
                            'mp3', None, 'copy', 'mute'),
                  ('mp4'): ('default', None, None, 'aac', None, 'ac3', None,
                            'mp3', 'opus', 'copy', 'mute'),
                  ('m4v'): ('default', None, None, 'aac', 'alac', None, None,
                            None, None, 'copy', 'mute'),
                  ('mkv'): ('default', 'wav', 'flac', 'aac', None, 'ac3',
                            'ogg', 'mp3', 'opus', 'copy', 'mute'),
                  ('webm'): ('default', None, None, None, None, None, 'ogg',
                             None, 'opus', 'copy', 'mute'),
                  ('wav'): (None, 'wav', None, None, None, None, None, None,
                            None, 'copy', None),
                  ('mp3'): (None, None, None, None, None, None, None, 'mp3',
                            None, 'copy', None),
                  ('ac3'): (None, None, None, None, None, 'ac3', None, None,
                            None, 'copy', None),
                  ('ogg'): (None, None, None, None, None, None, 'ogg', None,
                            'opus', 'copy', None),
                  ('flac'): (None, None, 'flac', None, None, None, None, None,
                             None, 'copy', None),
                  ('m4a'): (None, None, None, None, 'alac', None, None, None,
                            None, 'copy', None),
                  ('aac'): (None, None, None, 'aac', None, None, None, None,
                            None, 'copy', None),
                  ('opus'): (None, None, None, None, None, None, None, None,
                             'opus', 'copy', None),
                  }

    def __init__(self, parent, opt, maindata):
        """
        This is a child of `nb_Audio` of `AV_Conv` class-panel (parent)
        """
        self.parent = parent  # parent is the `nb_Audio` here.
        self.maindata = maindata  # data on MainFrame
        self.appdata = self.maindata.appdata
        self.opt = opt
        icons = self.maindata.icons

        if 'wx.svg' in sys.modules:  # only available in wx version 4.1 to up
            bmpapreview = get_bmp(icons['preview_audio'], ((16, 16)))
            bmpanalyzes = get_bmp(icons['volanalyze'], ((16, 16)))
            bmpasettings = get_bmp(icons['settings'], ((16, 16)))
            bmppeaklevel = get_bmp(icons['audiovolume'], ((16, 16)))
        else:
            bmpapreview = wx.Bitmap(icons['preview_audio'], wx.BITMAP_TYPE_ANY)
            bmpanalyzes = wx.Bitmap(icons['volanalyze'], wx.BITMAP_TYPE_ANY)
            bmpasettings = wx.Bitmap(icons['settings'], wx.BITMAP_TYPE_ANY)
            bmppeaklevel = wx.Bitmap(icons['audiovolume'], wx.BITMAP_TYPE_ANY)

        scrolled.ScrolledPanel.__init__(self, parent, -1,
                                        size=(1024, 1024),
                                        style=wx.TAB_TRAVERSAL
                                        | wx.BORDER_NONE,
                                        name="Audio Codecs scrolledpanel",
                                        )
        sizerbase = wx.BoxSizer(wx.VERTICAL)
        self.labinfo = wx.StaticText(self, wx.ID_ANY,
                                     label=_('Audio encoder settings, '
                                             'mapping and filters'),
                                     style=wx.ALIGN_CENTRE_HORIZONTAL)
        sizerbase.Add(self.labinfo, 0, wx.EXPAND | wx.ALL, 2)
        if self.appdata['ostype'] == 'Darwin':
            self.labinfo.SetFont(wx.Font(14, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        else:
            self.labinfo.SetFont(wx.Font(11, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        # sizerbase.Add((0, 15), 0)
        self.rdb_a = wx.RadioBox(self, wx.ID_ANY,
                                 (_("Audio Encoder")), size=(-1, -1),
                                 choices=list(AudioEncoders.ACODECS.keys()),
                                 majorDimension=0, style=wx.RA_SPECIFY_COLS
                                 )
        for n, v in enumerate(AudioEncoders.AV_FORMATS["mkv"]):
            if not v:  # disable only not compatible with mkv
                self.rdb_a.EnableItem(n, enable=False)

        sizerbase.Add(self.rdb_a, 0, wx.ALL | wx.CENTRE, 5)
        self.btn_set = wx.Button(self, wx.ID_ANY, _("Settings"),
                                 size=(-1, -1))
        self.btn_set.SetBitmap(bmpasettings, wx.LEFT)
        sizerbase.Add(self.btn_set, 0, wx.ALL | wx.CENTRE, 5)
        line1 = wx.StaticLine(self, wx.ID_ANY, pos=wx.DefaultPosition,
                              size=(-1, -1), style=wx.LI_HORIZONTAL,
                              name=wx.StaticLineNameStr
                              )
        sizerbase.Add(line1, 0, wx.ALL | wx.EXPAND, 15)
        sizproper = wx.BoxSizer(wx.HORIZONTAL)
        sizerbase.Add(sizproper, 0, wx.CENTRE)
        txtAinmap = wx.StaticText(self, wx.ID_ANY, _('Index Selection:'))
        sizproper.Add(txtAinmap, 0, wx.LEFT | wx.CENTRE, 20)
        self.cmb_A_inMap = wx.ComboBox(self, wx.ID_ANY,
                                       choices=['Auto', '1', '2', '3',
                                                '4', '5', '6', '7', '8', '9',
                                                '10', '11', '12', '13', '14',
                                                '15', '16',],
                                       size=(-1, -1), style=wx.CB_DROPDOWN
                                       | wx.CB_READONLY,
                                       )
        sizproper.Add(self.cmb_A_inMap, 0, wx.LEFT | wx.CENTRE, 5)
        txtAoutmap = wx.StaticText(self, wx.ID_ANY, _('Map:'))
        sizproper.Add(txtAoutmap, 0, wx.LEFT | wx.CENTRE, 20)
        self.cmb_A_outMap = wx.ComboBox(self, wx.ID_ANY,
                                        choices=['Auto', 'All', 'Index only'],
                                        size=(-1, -1),
                                        style=wx.CB_DROPDOWN
                                        | wx.CB_READONLY,
                                        )
        sizproper.Add(self.cmb_A_outMap, 0, wx.LEFT | wx.CENTRE, 5)
        self.btn_audio_preview = wx.Button(self, wx.ID_ANY,
                                           _("Preview"), size=(-1, -1))
        self.btn_audio_preview.SetBitmap(bmpapreview, wx.LEFT)
        sizproper.Add(self.btn_audio_preview, 0, wx.LEFT | wx.CENTRE, 20)
        line2 = wx.StaticLine(self, wx.ID_ANY, pos=wx.DefaultPosition,
                              size=(-1, -1), style=wx.LI_HORIZONTAL,
                              name=wx.StaticLineNameStr
                              )
        sizerbase.Add(line2, 0, wx.ALL | wx.EXPAND, 15)
        sizer_a_normaliz = wx.BoxSizer(wx.VERTICAL)
        sizerbase.Add(sizer_a_normaliz, 0, wx.EXPAND)
        normopt = [('Off'), ('PEAK'), ('RMS'), ('EBU R128 (Worst)'),
                   ('EBU R128 (High-Quality)')]
        self.rdbx_normalize = wx.RadioBox(self, wx.ID_ANY,
                                          (_("Normalization")), size=(-1, -1),
                                          choices=normopt,
                                          majorDimension=1,
                                          style=wx.RA_SPECIFY_ROWS,
                                          )
        sizer_a_normaliz.Add(self.rdbx_normalize, 0, wx.ALL | wx.CENTRE, 5)
        sizer_a_normaliz.Add((0, 10), 0)

        grid_peak = wx.FlexGridSizer(1, 4, 15, 4)
        sizer_a_normaliz.Add(grid_peak, 0, wx.ALL | wx.CENTRE, 5)
        self.btn_voldect = wx.Button(self, wx.ID_ANY,
                                     _("Volume detect"), size=(-1, -1))
        self.btn_voldect.SetBitmap(bmppeaklevel, wx.LEFT)
        grid_peak.Add(self.btn_voldect, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        self.btn_stat = wx.Button(self, wx.ID_ANY,
                                  _("Volume Statistics"), size=(-1, -1))
        self.btn_stat.SetBitmap(bmpanalyzes, wx.LEFT)
        grid_peak.Add(self.btn_stat, 0, wx.ALIGN_CENTER_VERTICAL, 0)

        self.lab_amplitude = wx.StaticText(self, wx.ID_ANY, _("Target level:"))
        grid_peak.Add(self.lab_amplitude, 0, wx.LEFT
                      | wx.ALIGN_CENTER_VERTICAL, 20)
        self.spin_target = FS.FloatSpin(self, wx.ID_ANY,
                                        min_val=-99.0, max_val=0.0,
                                        increment=0.5, value=-1.0,
                                        agwStyle=FS.FS_LEFT, size=(120, -1)
                                        )
        self.spin_target.SetFormat("%f"), self.spin_target.SetDigits(1)
        grid_peak.Add(self.spin_target, 0, wx.LEFT | wx.CENTRE, 2)

        self.lab_help_norm = wx.StaticText(self, wx.ID_ANY, (""),
                                           style=wx.ALIGN_CENTRE_HORIZONTAL)
        sizer_a_normaliz.Add(self.lab_help_norm, 0, wx.ALL | wx.CENTRE, 5)

        sizer_a_normaliz.Add((0, 15), 0)
        grid_ebu = wx.FlexGridSizer(1, 6, 0, 0)
        sizer_a_normaliz.Add(grid_ebu, 0, wx.ALL | wx.CENTRE, 5)
        lab_i = wx.StaticText(self, wx.ID_ANY, ("IL:"))
        grid_ebu.Add(lab_i, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        self.spin_i = FS.FloatSpin(self, wx.ID_ANY,
                                   min_val=-70.0, max_val=-5.0,
                                   increment=0.5, value=-16.0,
                                   agwStyle=FS.FS_LEFT, size=(120, -1)
                                   )
        self.spin_i.SetFormat("%f"), self.spin_i.SetDigits(1)
        grid_ebu.Add(self.spin_i, 0, wx.LEFT | wx.CENTRE, 5)

        lab_tp = wx.StaticText(self, wx.ID_ANY, ("TP:"))
        grid_ebu.Add(lab_tp, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 20)
        self.spin_tp = FS.FloatSpin(self, wx.ID_ANY,
                                    min_val=-9.0, max_val=0.0,
                                    increment=0.5, value=-1.5,
                                    agwStyle=FS.FS_LEFT, size=(120, -1)
                                    )
        self.spin_tp.SetFormat("%f"), self.spin_tp.SetDigits(1)
        grid_ebu.Add(self.spin_tp, 0, wx.LEFT | wx.CENTRE, 5)

        lab_lra = wx.StaticText(self, wx.ID_ANY, ("LRA:"))
        grid_ebu.Add(lab_lra, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 20)
        self.spin_lra = FS.FloatSpin(self, wx.ID_ANY,
                                     min_val=1.0, max_val=20.0,
                                     increment=0.5, value=11.0,
                                     agwStyle=FS.FS_LEFT, size=(120, -1)
                                     )
        self.spin_lra.SetFormat("%f"), self.spin_lra.SetDigits(1)
        grid_ebu.Add(self.spin_lra, 0, wx.LEFT | wx.CENTRE, 5)
        self.lab_help_ebu = wx.StaticText(self, wx.ID_ANY, (""),
                                          style=wx.ALIGN_CENTRE_HORIZONTAL)
        sizer_a_normaliz.Add(self.lab_help_ebu, 0, wx.ALL | wx.CENTRE, 5)

        if self.appdata['ostype'] == 'Darwin':
            self.lab_help_norm.SetFont(wx.Font(11, wx.DEFAULT,
                                               wx.NORMAL, wx.NORMAL))
            self.lab_help_ebu.SetFont(wx.Font(11, wx.DEFAULT,
                                              wx.NORMAL, wx.NORMAL))
        else:
            self.lab_help_norm.SetFont(wx.Font(8, wx.DEFAULT,
                                               wx.NORMAL, wx.NORMAL))
            self.lab_help_ebu.SetFont(wx.Font(8, wx.DEFAULT,
                                              wx.NORMAL, wx.NORMAL))
        self.SetSizer(sizerbase)  # set panel
        self.SetAutoLayout(1)
        self.SetupScrolling()

        tip = (_('Gets maximum volume and average volume data in dBFS, then '
                 'calculates the offset amount for audio normalization.'))
        self.btn_voldect.SetToolTip(tip)
        tip = (_('Limiter for the maximum peak level or the mean level '
                 '(when switch to RMS) in dBFS. From -99.0 to +0.0; default '
                 'for PEAK level is -1.0; default for RMS is -20.0'))
        self.spin_target.SetToolTip(tip)
        tip = (_('"Auto", lets FFmpeg select the audio stream to process '
                 '(usually the first audio stream). If the source file(s) '
                 'is just an audio track, it\'s recommend to always set this '
                 'control to "Auto."\n\n"1-16", if a video file contains '
                 'more than one audio stream, you can select a specific '
                 'one, e.g "1" for the first available audio stream, '
                 '"2" for the second audio stream, and so on.'))
        self.cmb_A_inMap.SetToolTip(tip)
        tip = (_('"Auto" keeps all audio streams and processes only the one '
                 'selected by the "Index Selection" control.\n\n'
                 '"All" processes all audio streams with the properties '
                 'of the one selected by the "Index Selection" control. '
                 'Don\'t use "All" to export audio files from videos.\n\n'
                 '"Index Only" processes only the audio stream selected by '
                 'the "Index Selection" control and removes all others '
                 'from the output video.'))
        self.cmb_A_outMap.SetToolTip(tip)
        tip = (_('Integrated Loudness Target in LUFS. '
                 'From -70.0 to -5.0, default is -16.0'))
        self.spin_i.SetToolTip(tip)
        tip = (_('Maximum True Peak in dBTP. From -1.5 '
                 'to +0.0, default is -2.0'))
        self.spin_tp.SetToolTip(tip)
        tip = (_('Loudness Range Target in LUFS. '
                 'From +1.0 to +20.0, default is +11.0'))
        self.spin_lra.SetToolTip(tip)
        tip = (_('Play the audio stream selected in the "Index Selection" '
                 'control. Using this function you can test the result of '
                 'audio normalization.'))
        self.btn_audio_preview.SetToolTip(tip)

        self.Bind(wx.EVT_RADIOBOX, self.on_getting_acodec, self.rdb_a)
        self.Bind(wx.EVT_BUTTON, self.on_setting_acodec, self.btn_set)
        self.Bind(wx.EVT_COMBOBOX, self.on_audio_index, self.cmb_A_inMap)
        self.Bind(wx.EVT_COMBOBOX, self.on_audio_mapping, self.cmb_A_outMap)
        self.Bind(wx.EVT_RADIOBOX, self.on_normalize, self.rdbx_normalize)
        self.Bind(wx.EVT_SPINCTRL, self.on_enter_gain, self.spin_target)
        self.Bind(wx.EVT_BUTTON, self.on_analyzes, self.btn_voldect)
        self.Bind(wx.EVT_BUTTON, self.on_show_vol_statistics, self.btn_stat)
    # ------------------------------------------------------------------#

    def audio_options(self):
        """
        Get audio parameters
        """
        if self.rdbx_normalize.GetSelection() == 0:
            del self.opt["PEAK"][:]
            del self.opt["RMS"][:]
            self.opt["EBU"] = ["", ""]

        if self.rdbx_normalize.GetSelection() == 1:  # is checked
            del self.opt["RMS"][:]
            self.opt["EBU"] = ["", ""]

        elif self.rdbx_normalize.GetSelection() == 2:
            del self.opt["PEAK"][:]
            self.opt["EBU"] = ["", ""]

        elif self.rdbx_normalize.GetSelection() == 3:
            del self.opt["PEAK"][:]
            del self.opt["RMS"][:]
            loud = (f'-filter:a: loudnorm=I={str(self.spin_i.GetValue())}'
                    f':LRA={str(self.spin_lra.GetValue())}'
                    f':TP={str(self.spin_tp.GetValue())}')
            self.opt["EBU"][1] = loud

        elif self.rdbx_normalize.GetSelection() == 4:
            del self.opt["PEAK"][:]
            del self.opt["RMS"][:]
            loud = (f'loudnorm=I={str(self.spin_i.GetValue())}:'
                    f'TP={str(self.spin_tp.GetValue())}:'
                    f'LRA={str(self.spin_lra.GetValue())}:'
                    f'print_format=summary'
                    )
            self.opt["EBU"][1] = loud

        return (f'{self.opt["AudioMap"][0]} '
                f'{self.opt["AudioCodec"][0]} '
                f'{self.opt["AudioCodec"][1]} '
                f'{self.opt["AudioBitrate"][1]} '
                f'{self.opt["AudioRate"][1]} '
                f'{self.opt["AudioChannel"][1]} '
                f'{self.opt["AudioDepth"][1]}'
                )
    # ------------------------------------------------------------------#

    def startup_one_time(self):
        """
        This method needs to be called only once when the
        parent `init` method is started, to create the
        required default dict keys.
        """
        self.opt["AudioIndex"] = ""
        self.opt["AudioMap"] = ["-map 0:a:?", ""]
        self.opt["PEAK"] = []
        self.opt["RMS"] = []
        self.opt["EBU"] = ["", ""]
        self.audio_default()
    # ------------------------------------------------------------------#

    def audio_default(self):
        """
        Set default audio parameters. This method is called
        whenever changes the video container selection.
        """
        self.rdb_a.SetStringSelection("Auto")
        self.opt["AudioCodStr"] = "Auto"
        self.opt["AudioCodec"] = ["", ""]
        self.opt["AudioBitrate"] = ["", ""]
        self.opt["AudioChannel"] = ["", ""]
        self.opt["AudioRate"] = ["", ""]
        self.opt["AudioDepth"] = ["", ""]
        self.btn_set.Disable()
        self.btn_set.SetBackgroundColour(wx.NullColour)
        # self.rdbx_normalize.Enable()
        self.cmb_A_outMap.SetSelection(0)
        self.cmb_A_inMap.SetSelection(0)
        self.cmb_A_outMap.Disable()
    # -------------------------------------------------------------------#

    def normalize_default(self, setoff='all'):
        """
        Reset normalization parameters on the audio properties.
        This method even is called by `MainFrame.switch_video_conv()`
        on start-up and when there are changing on `dragNdrop` panel.
        """
        del self.opt["PEAK"][:]
        del self.opt["RMS"][:]
        self.opt["EBU"] = ["", ""]

        if setoff == 'all':
            self.rdbx_normalize.SetSelection(0)
            self.spin_i.Disable(), self.spin_lra.Disable()
            self.spin_tp.Disable(), self.btn_voldect.Disable()
            self.spin_target.Disable(), self.btn_stat.Disable()
            self.spin_target.SetValue(-1.0)

        elif setoff == 'ebu':
            self.btn_voldect.Disable(), self.spin_target.Disable()
            self.btn_stat.Disable(), self.spin_i.Enable()
            self.spin_lra.Enable(), self.spin_tp.Enable()
            self.opt["EBU"][0] = self.rdbx_normalize.GetStringSelection()

        elif setoff == 'rms':
            self.spin_i.Disable(), self.spin_lra.Disable()
            self.spin_tp.Disable(), self.btn_voldect.Enable()
            self.spin_target.Enable(), self.btn_stat.Disable()
            self.spin_target.SetValue(-20.0)

        elif setoff == 'peak':
            self.spin_i.Disable(), self.spin_lra.Disable()
            self.spin_tp.Disable(), self.btn_voldect.Enable()
            self.spin_target.Enable(), self.btn_stat.Disable()
            self.spin_target.SetValue(-1.0)
    # ------------------------------------------------------------------#

    def on_audio_preview(self, fileget):
        """
        It allows a direct evaluation of the sound results given
        by the audio filters with the ability to playback even the
        selected audio streams through audio index mapping.
        """
        def _undetect():
            wx.MessageBox(_('Undetected volume values! Click the '
                            '"Volume detect" button to analyze '
                            'audio volume data.'),
                          'Videomass', wx.ICON_INFORMATION, self
                          )

        # fileget = self.parent.file_selection()
        if not fileget or not self.get_audio_stream(fileget):
            return None

        if self.cmb_A_inMap.GetValue() == 'Auto':
            idx = ''
        else:
            idx = f'-ast a:{str(int(self.cmb_A_inMap.GetValue()) - 1)}'

        if self.rdbx_normalize.GetSelection() == 0:
            afilter = ''

        elif self.rdbx_normalize.GetSelection() == 1:
            if self.btn_voldect.IsEnabled():
                return _undetect()
            afilter = f'-af {self.opt["PEAK"][fileget[1]][5].split()[1]}'

        elif self.rdbx_normalize.GetSelection() == 2:
            if self.btn_voldect.IsEnabled():
                return _undetect()
            afilter = f'-af {self.opt["RMS"][fileget[1]][5].split()[1]}'

        elif self.rdbx_normalize.GetSelection() in (3, 4):
            afilter = (f'-af loudnorm=I={str(self.spin_i.GetValue())}'
                       f':LRA={str(self.spin_lra.GetValue())}'
                       f':TP={str(self.spin_tp.GetValue())}')
        else:
            return None

        self.build_playfile_command(afilter,
                                    idx,
                                    self.maindata.file_src[fileget[1]]
                                    )
        return None
    # ------------------------------------------------------------------#

    def build_playfile_command(self, afilter, index, filepath):
        """
        Build ffplay command for playback
        """
        autoexit = '-autoexit' if self.maindata.autoexit else ''

        if self.maindata.checktimestamp:
            args = (f'-showmode waves {autoexit} -i "{filepath}" '
                    f'{self.maindata.time_seq} '
                    f'-vf "{self.maindata.cmdtimestamp}" {afilter} {index}')
        else:
            args = (f'-showmode waves {autoexit} -i "{filepath}" '
                    f'{self.maindata.time_seq} '
                    f'{afilter} {index}')
        try:
            with open(filepath, encoding='utf-8'):
                FilePlay(self.GetParent(), args)
        except IOError:
            wx.MessageBox(_("Invalid or unsupported file:  %s") % (filepath),
                          "Videomass", wx.ICON_EXCLAMATION, self)
    # ------------------------------------------------------------------#

    def get_audio_stream(self, fileselected):
        """
        Given a selected media file (object of type `file_selection()`),
        it evaluates whether it contains any audio streams and any
        indexes based on selected index (audio map).

        If there are no audio streams in the source file or the index
        does not match one of the existing audio streams, returns None.
        Return a list of audio indexes otherwise.

        See `on_audio_preview()` method for usage.

        """
        selected = self.maindata.data_files[fileselected[1]].get('streams')
        isaudio = [a for a in selected if 'audio' in a.get('codec_type')]
        idx = self.cmb_A_inMap.GetValue()

        if isaudio:
            streamindexes = list(x.get('index') for x in isaudio)
            if idx.isdigit():  # e.g user has selected an index (1-16)
                if not int(idx) - 1 in range(len(streamindexes)):
                    wx.MessageBox(_('The selected index does not '
                                    'exist in the source file:\n"{}"'
                                    ).format(fileselected[0]),
                                  'Videomass', wx.ICON_ERROR, self)
                    return None
            return streamindexes  # e.g user has selected 'Auto'
        else:
            wx.MessageBox(_('The source file does not contain any '
                            'audio stream:\n"{}"').format(fileselected[0]),
                          'Videomass', wx.ICON_ERROR, self)
        return None
    # ------------------------------------------------------------------#

    def set_audio_radiobox(self, event):
        """
        Container combobox sets compatible audio codecs
        for the selected format. See AV_FORMATS dict

        """
        if self.opt["Media"] == 'Video':
            if not self.opt["OutputFormat"]:  # Copy, enable all audio enc
                for n in range(self.rdb_a.GetCount()):
                    self.rdb_a.EnableItem(n, enable=True)
            else:
                for n, v in enumerate(AudioEncoders.AV_FORMATS[
                        self.opt["OutputFormat"]]):
                    if v:
                        self.rdb_a.EnableItem(n, enable=True)
                    else:
                        self.rdb_a.EnableItem(n, enable=False)
            self.rdb_a.SetSelection(0)

        if self.opt["Media"] == 'Audio':
            for n, v in enumerate(AudioEncoders.AV_FORMATS[
                    self.opt["OutputFormat"]]):
                if v:
                    self.rdb_a.EnableItem(n, enable=True)
                    # self.rdb_a.SetSelection(n)
                else:
                    self.rdb_a.EnableItem(n, enable=False)
            for x in range(self.rdb_a.GetCount()):
                if self.rdb_a.IsItemEnabled(x):
                    self.rdb_a.SetSelection(x)
                    break
            self.on_getting_acodec(self)
    # ------------------------------------------------------------------#

    def on_getting_acodec(self, event):
        """
        choosing an item on audio radiobox list, sets the
        audio format name and the appropriate command arg,
        (see ACODECS dict), resets the audio normalize and
        some `self.opt` keys.
        """
        audiocodec = self.rdb_a.GetStringSelection()

        def _param(enablenormalization, enablebuttonparameters):
            self.opt["AudioBitrate"] = ["", ""]
            self.opt["AudioChannel"] = ["", ""]
            self.opt["AudioRate"] = ["", ""]
            self.opt["AudioDepth"] = ["", ""]

            if enablenormalization:
                self.rdbx_normalize.Enable()
            else:
                self.rdbx_normalize.Disable()
            if enablebuttonparameters:
                self.btn_set.Enable()
                self.btn_set.SetBackgroundColour(wx.NullColour)
            else:
                self.btn_set.Disable()
                self.btn_set.SetBackgroundColour(wx.NullColour)

        # --------------------------------------------------------
        for k, v in AudioEncoders.ACODECS.items():
            if audiocodec in k:
                if audiocodec == "Auto":
                    self.audio_default()
                    self.rdbx_normalize.Enable()
                    self.opt["AudioCodec"] = ["", ""]

                elif audiocodec == "Copy":
                    self.rdbx_normalize.SetSelection(0)
                    self.on_normalize(self)
                    _param(False, False)
                    amap = f'-c:a:{self.opt["AudioMap"][1]}'
                    self.opt["AudioCodec"] = [amap, v]

                elif audiocodec == "No Audio":
                    self.rdbx_normalize.SetSelection(0)
                    self.on_normalize(self)
                    self.opt["AudioCodec"] = ["", v]
                    _param(False, False)
                    # break
                else:
                    _param(True, True)
                    amap = f'-c:a:{self.opt["AudioMap"][1]}'
                    self.opt["AudioCodec"] = [amap, v]

                self.opt["AudioCodStr"] = audiocodec

        if audiocodec == 'No Audio':  # audio Mapping disable
            self.cmb_A_inMap.Disable()
            self.cmb_A_outMap.Disable()
            self.opt["AudioMap"] = ["", ""]
            self.opt["AudioIndex"] = ""
        else:
            self.cmb_A_inMap.Enable()
            self.on_audio_index(self)
    # -------------------------------------------------------------------#

    def on_setting_acodec(self, event):
        """
        Event by Audio options button. Set audio codec string and audio
        command string and pass it to audio_dialogs method.
        """
        pcm = ["pcm_s16le", "pcm_s24le", "pcm_s32le"]

        if self.opt["AudioCodec"][1] in pcm:
            self.audio_dialog(self.opt["AudioCodStr"],
                              f'{self.opt["AudioCodStr"]} Audio Settings')
        else:
            self.audio_dialog(self.opt["AudioCodStr"],
                              f'{self.opt["AudioCodStr"]} Audio Settings')
    # -------------------------------------------------------------------#

    def audio_dialog(self, codecname, caption):
        """
        Given an audio codec specified by `codecname`,
        show a dialog for setting audio parameters
        related to the codec.
        """
        with AudioProperties(self,
                             codecname,
                             caption,
                             self.opt["AudioRate"],
                             self.opt["AudioDepth"],
                             self.opt["AudioBitrate"],
                             self.opt["AudioChannel"],
                             ) as properties:
            if properties.ShowModal() == wx.ID_OK:
                aparam = properties.getvalue()
            else:
                return

        self.opt["AudioChannel"] = aparam[0]
        self.opt["AudioRate"] = aparam[1]
        self.opt["AudioBitrate"] = aparam[2]
        if codecname == 'PCM':  # wav, aiff, etc
            amap = f'-c:a:{self.opt["AudioMap"][1]}'
            if 'Auto' in aparam[3][0]:  # [3] bit depth tuple
                self.opt["AudioCodec"] = [amap, "pcm_s16le"]
            else:
                self.opt["AudioCodec"] = [amap, aparam[3][1]]
            self.opt["AudioDepth"] = (f"{aparam[3][0]}", '')  # none
        else:  # all, except PCM
            self.opt["AudioDepth"] = aparam[3]

        count = 0
        for descr in (self.opt["AudioRate"],
                      aparam[3],
                      self.opt["AudioBitrate"],
                      self.opt["AudioChannel"],
                      ):
            if descr[1]:
                count += 1

        if count == 0:
            self.btn_set.SetBackgroundColour(wx.NullColour)
        else:
            self.btn_set.SetBackgroundColour(wx.Colour(AudioEncoders.VIOLET))
        return
    # ------------------------------------------------------------------#

    def on_audio_index(self, event):
        """
        Set a specific index from audio streams.
        See: <http://ffmpeg.org/ffmpeg.html#Advanced-options>
        """
        if self.cmb_A_inMap.GetValue() == 'Auto':
            self.cmb_A_outMap.Disable()
            self.opt["AudioIndex"] = ''
        else:
            self.cmb_A_outMap.Enable()
            idx = str(int(self.cmb_A_inMap.GetValue()) - 1)
            self.opt["AudioIndex"] = f'-map 0:a:{idx}'

        if self.rdbx_normalize.GetSelection() in [1, 2]:
            if not self.btn_voldect.IsEnabled():
                self.btn_voldect.Enable()

        self.on_audio_mapping(None)
    # ------------------------------------------------------------------#

    def on_audio_mapping(self, event):
        """
        Set the mapping of the audio streams.
        """
        index = self.cmb_A_inMap.GetValue()
        sel = self.cmb_A_outMap.GetValue()
        idx = '' if index == 'Auto' else str(int(index) - 1)

        if sel == 'Auto':
            if self.opt["Media"] == 'Video':
                self.opt["AudioMap"] = ['-map 0:a:?', idx]

            elif self.opt["Media"] == 'Audio':
                self.opt["AudioMap"] = [f'-map 0:a:{idx}?', '']

        elif sel == 'All':
            self.opt["AudioMap"] = ['-map 0:a:?', '']

        elif sel == 'Index only':
            self.opt["AudioMap"] = [f'-map 0:a:{idx}?', '']

        if self.opt["AudioCodec"][0]:
            self.opt["AudioCodec"][0] = f"-c:a:{self.opt['AudioMap'][1]}"
    # ------------------------------------------------------------------#

    def on_normalize(self, event):
        """
        Enable or disable functionality for volume normalization.
        """
        msg_1 = (_('PEAK level normalization, which will produce '
                   'a maximum peak level equal to the set target level.'
                   ))
        msg_2 = (_('RMS-based normalization, which according to '
                   'mean volume calculates the amount of gain to reach same '
                   'average power signal.'
                   ))
        msg_3 = (_('Loudnorm normalization. Normalizes '
                   'the perceived loudness using the \"loudnorm\" filter, '
                   'which implements the EBU R128 algorithm.\n'
                   'Ideal for live normalization. It produces worse results '
                   'than the High-quality two-pass Loudnorm normalization, '
                   'but is faster.'
                   ))
        msg_4 = (_('High-quality two-pass Loudnorm normalization. '
                   'Normalizes the perceived loudness using the '
                   '\"loudnorm\" filter, which implements\nthe EBU R128 '
                   'algorithm. Ideal for postprocessing.'
                   ))
        if self.rdbx_normalize.GetSelection() == 0:
            self.normalize_default('all')
            self.lab_help_norm.SetLabel(''), self.lab_help_ebu.SetLabel('')

        if self.rdbx_normalize.GetSelection() == 1:  # is checked
            self.normalize_default('peak')
            self.lab_help_norm.SetLabel(msg_1), self.lab_help_ebu.SetLabel('')

        elif self.rdbx_normalize.GetSelection() == 2:
            self.normalize_default('rms')
            self.lab_help_norm.SetLabel(msg_2), self.lab_help_ebu.SetLabel('')

        elif self.rdbx_normalize.GetSelection() == 3:
            self.lab_help_ebu.SetLabel(msg_3), self.lab_help_norm.SetLabel('')
            self.normalize_default('ebu')

        elif self.rdbx_normalize.GetSelection() == 4:
            self.lab_help_ebu.SetLabel(msg_4), self.lab_help_norm.SetLabel('')
            self.normalize_default('ebu')
        self.Layout()
    # ------------------------------------------------------------------#

    def on_enter_gain(self, event):
        """
        when spin_amplitude is changed enable 'Volumedetect' to
        update new incomming

        """
        if not self.btn_voldect.IsEnabled():
            self.btn_voldect.Enable()
    # ------------------------------------------------------------------#

    def on_analyzes(self, event):
        """
        Clicking on the "Detect Volume" button performs the
        PEAK/RMS-based volume detection and analysis process
        required to calculate the offset for audio volume
        normalization in dBFS:
            - PEAK-based Analyzes, get the MAXIMUM peak level data.
            - RMS-based Analyzes, get the MEAN peak level data.
        <https://superuser.com/questions/323119/how-can-i-normalize-audio-
        using-ffmpeg?utm_medium=organic>
        """
        if not self.maindata.data_files:
            wx.MessageBox(_("You need to import at least one file"),
                          'Videomass', wx.ICON_INFORMATION, self)
            return

        data = volume_detect_process(self.maindata.file_src,
                                     self.maindata.time_seq,  # from -ss to -t
                                     self.opt["AudioIndex"],
                                     parent=self.GetParent(),
                                     )
        if data[1]:  # see `volume_detect_process` in `io_tools`
            return

        if self.rdbx_normalize.GetSelection() == 1:  # PEAK
            target = "PEAK"
        elif self.rdbx_normalize.GetSelection() == 2:  # RMS
            target = "RMS"
        else:
            return

        del self.opt["PEAK"][:]
        del self.opt["RMS"][:]

        gain = self.spin_target.GetValue()
        for filename, vol in zip(self.maindata.file_src, data[0]):
            dataref = get_volume_data(filename,
                                      vol,
                                      gain=gain,
                                      target=target,
                                      audiomap=self.opt["AudioMap"][1],
                                      )
            self.opt[target].append(dataref)

        self.btn_voldect.Disable()
        self.btn_stat.Enable()
    # ------------------------------------------------------------------#

    def on_show_vol_statistics(self, event):
        """
        Show a wx.ListCtrl dialog with volumedetect data
        """
        if self.maindata.audivolnormalize:
            self.maindata.audivolnormalize.Raise()
            return

        if self.rdbx_normalize.GetSelection() == 1:  # PEAK
            title = _('PEAK-based volume statistics')
        elif self.rdbx_normalize.GetSelection() == 2:  # RMS
            title = _('RMS-based volume statistics')
        else:
            return

        if self.btn_voldect.IsEnabled():
            self.on_analyzes(self)

        lev = self.opt["RMS"] if not self.opt["PEAK"] else self.opt["PEAK"]
        self.maindata.audivolnormalize = AudioVolNormal(title,
                                                        lev,
                                                        self.appdata['ostype'],
                                                        )
        self.maindata.audivolnormalize.Show()
    # ------------------------------------------------------------------#



# -*- coding: UTF-8 -*-
"""
Name: avconv_cmd_line.py
Porpose: Shows the output raw command lines.
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2025 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Aug.13.2025
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import re
import wx
from videomass.vdms_dialogs.widget_utils import NormalTransientPopup


class Raw_Cmd_Line(wx.Dialog):
    """
    This class represents a modal dialog box
    to display output command lines.
    """
    get = wx.GetApp()
    APPDATA = get.appset
    PASS_1 = _("First pass command")
    PASS_2 = _("Second pass command")
    LGREEN = '#52ee7d'
    BLACK = '#1f1f1f'
    # ------------------------------------------------------------------

    def __init__(self, parent, *args):
        """
        `args` class tuple, contain from 0 to 1 items.
        """
        self.charsize = None
        self.cmd1 = args[0]
        self.cmd2 = args[1] if len(args) > 1 else ''

        wx.Dialog.__init__(self, parent, -1,
                           _("Videomass - RAW Commands"),
                           style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        size_base = wx.BoxSizer(wx.VERTICAL)

        boxshow = wx.BoxSizer(wx.HORIZONTAL)
        size_base.Add(boxshow, 0, wx.ALL, 5)

        btn_readme = wx.Button(self, wx.ID_ANY, _("Read me"), size=(-1, -1))
        btn_readme.SetBackgroundColour(wx.Colour(Raw_Cmd_Line.LGREEN))
        btn_readme.SetForegroundColour(wx.Colour(Raw_Cmd_Line.BLACK))
        boxshow.Add(btn_readme, 0,)

        size_base.Add((0, 15), 0)
        labpass1 = wx.StaticText(self, wx.ID_ANY, Raw_Cmd_Line.PASS_1)
        size_base.Add(labpass1, 0, wx.ALL | wx.EXPAND, 5)
        self.pass_1_cmd = wx.TextCtrl(self, wx.ID_ANY, "",
                                      style=wx.TE_MULTILINE
                                      | wx.TE_READONLY
                                      | wx.TE_RICH2,
                                      )
        size_base.Add(self.pass_1_cmd, 1, wx.ALL | wx.EXPAND, 5)

        size_base.Add((0, 15), 0)
        labpass2 = wx.StaticText(self, wx.ID_ANY, Raw_Cmd_Line.PASS_2)
        size_base.Add(labpass2, 0, wx.ALL | wx.EXPAND, 5)
        self.pass_2_cmd = wx.TextCtrl(self, wx.ID_ANY, "",
                                      style=wx.TE_MULTILINE
                                      | wx.TE_READONLY
                                      | wx.TE_RICH2,
                                      )
        size_base.Add(self.pass_2_cmd, 1, wx.ALL | wx.EXPAND, 5)
        minsize = (800, 550)

        # ----- confirm buttons section
        btn_close = wx.Button(self, wx.ID_CLOSE, "")
        btngrid = wx.FlexGridSizer(1, 1, 0, 0)
        btngrid.Add(btn_close, 0, wx.LEFT, 5)
        size_base.Add(btngrid, flag=wx.ALL
                      | wx.ALIGN_RIGHT
                      | wx.RIGHT, border=5
                      )
        # ------ Set Layout
        self.SetMinSize(minsize)
        self.SetSizer(size_base)
        self.Fit()
        self.Layout()

        # ----------------------Binder (EVT)----------------------#
        self.Bind(wx.EVT_BUTTON, self.on_help, btn_readme)
        self.Bind(wx.EVT_BUTTON, self.on_close, btn_close)
        self.Bind(wx.EVT_CLOSE, self.on_close)  # for caption

        self.textstyle()
    # ------------------------------------------------------------------#

    def textstyle(self):
        """
        Populate TextCtrls
        """
        if Raw_Cmd_Line.APPDATA['ostype'] == 'Darwin':
            self.charsize = 12
        else:
            self.charsize = 10

        self.pass_1_cmd.SetFont(wx.Font(self.charsize, wx.FONTFAMILY_TELETYPE,
                                        wx.NORMAL, wx.NORMAL))
        self.pass_2_cmd.SetFont(wx.Font(self.charsize, wx.FONTFAMILY_TELETYPE,
                                        wx.NORMAL, wx.NORMAL))
        backgrnd = '#232424'  # DARK GREY
        foregrnd = '#cfcfc2'  # MATTE WHITE

        self.pass_1_cmd.SetBackgroundColour(backgrnd)
        self.pass_1_cmd.Clear()
        self.pass_1_cmd.SetDefaultStyle(wx.TextAttr(foregrnd))
        self.pass_1_cmd.AppendText(self.cmd1)  # command 1

        self.pass_2_cmd.SetBackgroundColour(backgrnd)
        self.pass_2_cmd.Clear()
        self.pass_2_cmd.SetDefaultStyle(wx.TextAttr(foregrnd))
        self.pass_2_cmd.AppendText(self.cmd2)  # command 2

        self.code_highlight()
    # --------------------------------------------------------------#

    def code_highlight(self):
        """
        event on button help
        """
        bold = wx.Font(self.charsize,
                       wx.FONTFAMILY_TELETYPE, wx.NORMAL, wx.BOLD)

        colours = {'green': wx.TextAttr('#49eb49', font=bold),
                   'blue': wx.TextAttr('#118db5', font=bold),
                   'azure': wx.TextAttr('#8AB8E6', font=bold),
                   'grey': wx.TextAttr('#7a7c7d', font=bold),
                   'yellow': wx.TextAttr('#fdbc4b', font=bold),
                   'violet': wx.TextAttr('#8e44ad', font=bold),
                   'orange': wx.TextAttr('#f67400', font=bold),
                   'red': wx.TextAttr(wx.RED, font=bold),
                   'white': wx.TextAttr(wx.WHITE, font=bold),
                   }
        keyflags = {(' -map', ' -map_chapters ',
                     ' -map_metadata ', ' verbose '): colours['azure'],
                    (' -c:v', ' -c:a', ' -c:s', ' -an', ' -vn',
                     ' -sn', ' -dn'): colours['orange'],
                    (' -i ',): colours['white'],
                    (' -filter:v', ' -filter:a', ' -vf'):
                        colours['violet'],
                    ('volume=', 'loudnorm=', 'eq=', 'vidstabdetect=',
                     'vidstabtransform=', 'nlmeans=', 'hqdn3d=', 'w3fdif=',
                     'yadif=', 'interlace=', 'transpose=', 'crop=', 'scale=',
                     'setdar=', 'setsar=', 'unsharp=',
                     ' info '): colours['green'],
                    (' -pass ', '.mkv', '.mp4', '.webm', '.avi',
                     '.m4v', '.ogg', '.wav', '.mp3', '.ac3',
                     '.flac', '.m4a', '.aac', '.opus', 'matroska',
                     'null', 'NUL', '-f null', ' warning '): colours['yellow'],
                    (' -aspect', ' -r', ' -movflags',
                     ' debug '): colours['blue'],
                    (' -y ', ' -stats ', ' -hide_banner ',
                     ' -loglevel '): colours['grey'],
                    (r'\<\?\>', ' error '): colours['red'],
                    }
        for key in keyflags:
            for flag in key:
                matches1 = re.finditer(flag, self.cmd1)  # iterator
                matches2 = re.finditer(flag, self.cmd2)  # iterator

                for idx in matches1:
                    self.pass_1_cmd.SetStyle(idx.start(),
                                             idx.end(),
                                             keyflags.get(key))
                for idx in matches2:
                    self.pass_2_cmd.SetStyle(idx.start(),
                                             idx.end(),
                                             keyflags.get(key))

    # ---------------------Callbacks (event handler)--------------------#

    def on_help(self, event):
        """
        event on button help
        """
        msg = (_("The RAW commands reflect the user settings made on\n"
                 "AV-Conversions interface and apply only to the individual\n"
                 "selected file.\n\n"
                 "Please note that using EBU R128 (High-Quality), any <?>\n"
                 "markers highlighted in red should be replaced with the\n"
                 "values given by the output of the first command."))
        win = NormalTransientPopup(self,
                                   wx.SIMPLE_BORDER,
                                   msg,
                                   Raw_Cmd_Line.LGREEN,
                                   Raw_Cmd_Line.BLACK)

        # Show the popup right below or above the button
        # depending on available screen space...
        btn = event.GetEventObject()
        pos = btn.ClientToScreen((0, 0))
        sz = btn.GetSize()
        win.Position(pos, (0, sz[1]))

        win.Popup()
    # --------------------------------------------------------------#

    def on_close(self, event):
        """
        Destroy this dialog
        """
        self.Destroy()
        # event.Skip()



# -*- coding: UTF-8 -*-
"""
FileName: miscell.py
Porpose: Contains various functionalities for A/V Conversions
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2025 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Aug.07.2025
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import wx


class Miscellaneous(wx.Panel):
    """
    This Panel implements various functionalities
    controls for A/V Conversions.
    """
    # Namings in the subtitle codec selection on subtitle radio box:
    SCODECS = {('Auto'): (""),
               ('MOV_TEXT'): ("mov_text"),
               ('SUBRIP'): ("subrip"),
               ('WEBVTT'): ("webvtt"),
               ('ASS'): ("alac"),
               ('SSA'): ("ac3"),
               ('SRT'): ("libvorbis"),
               ('Copy'): ("copy"),
               ('No Subtitles'): ("-sn")
               }
    # compatibility between video formats and related subtitle formats:
    S_FORMATS = {('avi'): ('default', None, None, None, None, None, None,
                           'copy', 'No Subtitles'),
                 ('mp4'): ('default', 'mov_text', None, None, None, None, None,
                           'copy', 'No Subtitles'),
                 ('m4v'): ('default', 'mov_text', None, None, None, None, None,
                           'copy', 'No Subtitles'),
                 ('mkv'): ('default', None, 'subrip', 'webvtt', 'ass', 'ssa',
                           'srt', 'copy', 'No Subtitles'),
                 ('webm'): ('default', None, None, 'webvtt', None, None, None,
                            'copy', 'No Subtitles'),
                 }

    def __init__(self, parent, opt):
        """
        This is a child of `nb_misc` of AV_Conv` class-panel (parent).
        """
        get = wx.GetApp()
        self.appdata = get.appset
        self.parent = parent  # parent is the `nb_misc` here.
        self.opt = opt

        wx.Panel.__init__(self, parent, -1,
                          style=wx.TAB_TRAVERSAL
                          | wx.BORDER_NONE,
                          name="Miscellaneous panel",
                          )
        sizerbase = wx.BoxSizer(wx.VERTICAL)
        labsub = wx.StaticText(self, wx.ID_ANY, label="",
                               style=wx.ALIGN_CENTRE_HORIZONTAL)
        sizerbase.Add(labsub, 0, wx.EXPAND | wx.ALL, 2)
        if self.appdata['ostype'] == 'Darwin':
            labsub.SetFont(wx.Font(14, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        else:
            labsub.SetFont(wx.Font(11, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        labsub.SetLabel(_("Subtitle Properties"))
        self.rdb_s = wx.RadioBox(self, wx.ID_ANY,
                                 (_("Subtitle Encoder")), size=(-1, -1),
                                 choices=list(Miscellaneous.SCODECS.keys()),
                                 majorDimension=0, style=wx.RA_SPECIFY_COLS
                                 )
        for n, v in enumerate(Miscellaneous.S_FORMATS["mkv"]):
            if not v:  # disable only not compatible with mkv
                self.rdb_s.EnableItem(n, enable=False)
        sizerbase.Add(self.rdb_s, 0, wx.ALL | wx.CENTRE, 5)
        line1 = wx.StaticLine(self, wx.ID_ANY, pos=wx.DefaultPosition,
                              size=(-1, -1), style=wx.LI_HORIZONTAL,
                              name=wx.StaticLineNameStr
                              )
        sizerbase.Add(line1, 0, wx.ALL | wx.EXPAND, 15)
        gridsub = wx.BoxSizer(wx.HORIZONTAL)
        sizerbase.Add(gridsub, 0, wx.ALL | wx.CENTRE, 5)
        msg = _('Index Selection:')
        txtSubmap = wx.StaticText(self, wx.ID_ANY, (msg))
        gridsub.Add(txtSubmap, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        self.cmb_Submap = wx.ComboBox(self, wx.ID_ANY,
                                      choices=['All', '1', '2', '3',
                                               '4', '5', '6', '7', '8', '9',
                                               '10', '11', '12', '13', '14',
                                               '15', '16',],
                                      size=(120, -1), style=wx.CB_DROPDOWN
                                      | wx.CB_READONLY,
                                      )
        gridsub.Add(self.cmb_Submap, 0, wx.LEFT | wx.CENTRE, 5)
        line2 = wx.StaticLine(self, wx.ID_ANY, pos=wx.DefaultPosition,
                              size=(-1, -1), style=wx.LI_HORIZONTAL,
                              name=wx.StaticLineNameStr
                              )
        sizerbase.Add(line2, 0, wx.ALL | wx.EXPAND, 15)
        sizerbase.Add((0, 25), 0)
        labdata = wx.StaticText(self, wx.ID_ANY, label="",
                                style=wx.ALIGN_CENTRE_HORIZONTAL)
        sizerbase.Add(labdata, 0, wx.EXPAND | wx.ALL, 2)
        if self.appdata['ostype'] == 'Darwin':
            labdata.SetFont(wx.Font(14, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        else:
            labdata.SetFont(wx.Font(11, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        labdata.SetLabel(_("Chapters and Metadata"))
        boxmetad = wx.BoxSizer(wx.HORIZONTAL)
        sizerbase.Add(boxmetad, 0, wx.ALL | wx.CENTRE, 5)
        self.ckbx_chap = wx.CheckBox(self, wx.ID_ANY, (_('Copy Chapters')))
        boxmetad.Add(self.ckbx_chap, 0, wx.LEFT, 5)
        msg = _('Copy Metadata')
        self.ckbx_metad = wx.CheckBox(self, wx.ID_ANY, (msg))
        boxmetad.Add(self.ckbx_metad, 0, wx.LEFT, 20)
        self.SetSizerAndFit(sizerbase)
        tip = (_('Select "All" to include any possible subtitle stream in the '
                 'output video.\n\nSelect "1-16" to indexing the desired '
                 'subtitle stream and exclude all others, e.g "1" for the '
                 'first available subtitle stream, "2" for the second '
                 'subtitle stream, and so on.'))
        self.cmb_Submap.SetToolTip(tip)
        tip = (_('Copy the chapter markers as is from source file.'))
        self.ckbx_chap.SetToolTip(tip)
        tip = (_('Copy all incoming metadata from source file, such as '
                 'audio/video tags, titles, unique marks, and so on.'))
        self.ckbx_metad.SetToolTip(tip)

        self.Bind(wx.EVT_RADIOBOX, self.on_sub_enc, self.rdb_s)
        self.Bind(wx.EVT_CHECKBOX, self.on_chapters, self.ckbx_chap)
        self.Bind(wx.EVT_COMBOBOX, self.on_sub_map, self.cmb_Submap)
        self.Bind(wx.EVT_CHECKBOX, self.on_metadata, self.ckbx_metad)

        self.default()

    def default(self):
        """
        Reset all controls to default
        """
        self.cmb_Submap.SetSelection(0), self.on_sub_map(None)
        self.ckbx_chap.SetValue(True), self.on_chapters(None)
        self.ckbx_metad.SetValue(True), self.on_metadata(None)
    # ------------------------------------------------------------------#

    def set_subt_radiobox(self):
        """
        Sets compatible subtitle codec for the selected video format.
        See `S_FORMATS` dict on this class. This method is called
        changing video format (container)
        This method is called by av_conversion.py file only.
        """
        if self.opt["Media"] == 'Audio':
            self.rdb_s.Disable(), self.cmb_Submap.Disable(),
            self.ckbx_chap.Disable()
            return
        self.rdb_s.Enable(), self.cmb_Submap.Enable()
        self.ckbx_chap.Enable()

        if not self.opt["OutputFormat"]:  # in Copy, enable all audio enc
            for n in range(self.rdb_s.GetCount()):
                self.rdb_s.EnableItem(n, enable=True)
            self.cmb_Submap.Enable()
            self.cmb_Submap.SetSelection(0)
            self.rdb_s.SetSelection(0)
            self.opt["SubtitleMap"] = '-map 0:s?'
            self.opt["SubtitleEnc"] = ''
            return

        if not [k for k in Miscellaneous.S_FORMATS if
                self.opt["OutputFormat"] == k]:
            return

        for n, v in enumerate(Miscellaneous.S_FORMATS[
                self.opt["OutputFormat"]]):
            if v:
                self.rdb_s.EnableItem(n, enable=True)
            else:
                self.rdb_s.EnableItem(n, enable=False)

        if self.opt["OutputFormat"] in ('mp4', 'm4v'):
            self.cmb_Submap.SetSelection(0)
            self.cmb_Submap.Disable()
            self.opt["SubtitleMap"] = '-map 0:s?'
            self.rdb_s.SetSelection(1)
            subcodec = self.rdb_s.GetStringSelection()
            codec = Miscellaneous.SCODECS[subcodec]
            self.opt["SubtitleEnc"] = f'-c:s {codec}'

        elif self.opt["OutputFormat"] == 'avi':
            self.cmb_Submap.SetSelection(0)
            self.cmb_Submap.Disable()
            self.opt["SubtitleMap"] = ''
            self.rdb_s.SetSelection(8)
            self.opt["SubtitleEnc"] = '-sn'
        else:
            self.cmb_Submap.Enable()
            self.cmb_Submap.SetSelection(0)
            self.opt["SubtitleMap"] = '-map 0:s?'
            self.rdb_s.SetSelection(0)
            self.opt["SubtitleEnc"] = ''

        return
    # ------------------------------------------------------------------#

    def on_sub_enc(self, event):
        """
        Event on subtitle radiobox control
        """
        subcodec = self.rdb_s.GetStringSelection()
        if subcodec == 'No Subtitles':
            self.opt["SubtitleEnc"] = '-sn'
            self.opt["SubtitleMap"] = ''
            self.cmb_Submap.Disable()
            return
        if subcodec == 'MOV_TEXT':
            self.cmb_Submap.SetSelection(0)
            self.cmb_Submap.Disable()
            self.opt["SubtitleMap"] = '-map 0:s?'
            return
        if not self.cmb_Submap.IsEnabled():
            self.cmb_Submap.Enable()

        smap = self.cmb_Submap.GetValue()
        if smap.isdigit():
            idx = f':{str(int(smap) - 1)}'
        else:
            idx = ''

        if subcodec == 'Auto':
            self.opt["SubtitleEnc"] = ''
        elif subcodec == 'Copy':
            self.opt["SubtitleEnc"] = f'-c:s{idx} copy'
        else:
            codec = Miscellaneous.SCODECS[subcodec]
            self.opt["SubtitleEnc"] = f'-c:s{idx} {codec}'
    # ------------------------------------------------------------------#

    def on_sub_map(self, event):
        """
        Event on combobox list for subtitles
        """
        smap = self.cmb_Submap.GetValue()
        if smap == 'All':
            self.opt["SubtitleMap"] = '-map 0:s?'
        else:
            self.opt["SubtitleMap"] = f'-map 0:s:{str(int(smap) - 1)}'

        self.on_sub_enc(None)
    # ------------------------------------------------------------------#

    def on_chapters(self, event):
        """
        Event on enabling/disabling map chapters
        """
        val = self.ckbx_chap.GetValue()
        self.opt["Chapters"] = '-map_chapters 0' if val else '-map_chapters -1'
    # ------------------------------------------------------------------#

    def on_metadata(self, event):
        """
        Event on enabling/disabling metadata
        """
        val = self.ckbx_metad.GetValue()
        self.opt["MetaData"] = '-map_metadata 0' if val else '-map_metadata -1'
    # ------------------------------------------------------------------#



# -*- coding: UTF-8 -*-
"""
File Name: make_filelog.py
Porpose: log file generator
Compatibility: Python3, Python2
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2025 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Sep.19.2025
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""

import time
import os


def tolog(info, logfile, sep=False, wdate=False, txtenc="utf-8"):
    """
    This function writes log events as information messages
    to a given `logfile` during the processes.
    """
    if sep:
        line = '\n' + '-' * 80 + '\n'
    else:
        line = '\n'

    if wdate:
        curdate = time.strftime("%c")  # date/time
        strdate = f'DATE: {curdate}\n'
    else:
        strdate = ''

    apnd = f'{line}{strdate}{info}\n'

    with open(logfile, "a", encoding=txtenc) as log:
        log.write(apnd)
# ----------------------------------------------------------------#


def make_log_template(logname, logdir, mode="a", txtenc="utf-8"):
    """
    Most log files are initialized from a template
    before starting a process and writing status
    messages to a given log file.

    - logname,  example: `mylog.log`
    - logdir, log files location

    Returns an absolute/relative pathname of the logfile.
    """
    sep = '=' * 80
    current_date = time.strftime("%c")  # date/time
    logfile = os.path.join(logdir, logname)

    with open(logfile, mode, encoding=txtenc) as log:
        log.write(f"""{sep}

[PROGRAM NAME]: Videomass

[SESSION DATE]: {current_date}

[LOGFILE LOCATION]: "{logfile}"
""")
    return logfile
